--
-- TurnLightMotorized
-- Turn-Light Class for all Motorized Vehicles
--
-- @author  Jack ? / modified/excluded Hoschiander
-- @date  25/04/09
--
-- Copyright (C) Jack

TurnLightMotorized = {};

function TurnLightMotorized.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;


function TurnLightMotorized:load(xmlFile)

-- Bremslichter
    self.numbrakelights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.brakelights#count"), 0);
    self.brakelights = {};
    for i=1, self.numbrakelights do
        local brakelightnamei = string.format("vehicle.brakelights.brakelight" .. "%d", i);
        self.brakelights[i] = Utils.indexToObject(self.components, getXMLString(xmlFile, brakelightnamei .. "#index"));
        setVisibility(self.brakelights[i], false);
    end;

    self.brakelightsActive = false;
    self.braking = false;
    self.attached = true;
    self.isTurnLight = true;
    self.delayOff = 0;
    self.delayOffLeft = 0;
    self.delayOffRight = 0;

    self.numWorklights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.workLights#count"), 0);
    self.worklights = {};
    for i=1, self.numWorklights do
        local worklightnamei3 = string.format("vehicle.workLights.workLight" .. "%d", i);
        self.worklights[i] = Utils.indexToObject(self.rootNode, getXMLInt(xmlFile, worklightnamei3 .. "#index"));
        setVisibility(self.worklights[i], false);
    end;	
	self.worklightsActive = false;	
	
	self.numrearlights = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.rearlights#count"), 0);
    self.rearlights = {};
    for i=1, self.numrearlights do
        local rearlightnamei3 = string.format("vehicle.rearlights.rearlight" .. "%d", i);
        self.rearlights[i] = Utils.indexToObject(self.rootNode, getXMLInt(xmlFile, rearlightnamei3 .. "#index"));
        setVisibility(self.rearlights[i], false);
    end;	
	self.rearlightsActive = false;	
end;

function TurnLightMotorized:mouseEvent(posX, posY, isDown, isUp, button)
end;

function TurnLightMotorized:keyEvent(unicode, sym, modifier, isDown)


--[[Rueckfahrlicht start]]----------------------------------------------------
	if isDown and sym == Input.KEY_l then	
		self.worklightsActive = not self.worklightsActive;
		self.reverseLight = not self.reverseLight;
	end;

end;





function TurnLightMotorized:update(dt)

--print (self.isMotorStarted);


	-- Bremslicht
		local inputAxisY = InputBinding.getDigitalInputAxis(InputBinding.AXIS_FORWARD);

		if self.isEntered ~= nil then
		if self.brakelightsActive then
			for i=1, self.numbrakelights do
				local brakelight = self.brakelights[i];
				setVisibility(brakelight, self.brakelightsActive);
			end;
		else
			for i=1, self.numbrakelights do
				local brakelight = self.brakelights[i];
				setVisibility(brakelight, self.brakelightsActive, false);
			end;
		end;
		if ((self.movingDirection*self.lastSpeed*(-inputAxisY)) < -0.001 or self.movingDirection == 0) and (self.isEntered or self.isMotorStarted) then
			self.brakelightsActive = true;
			self.braking = true;
		else
			self.brakelightsActive = false;
			self.braking = false;
		end;
	end;

	if not self.reverseLight then
		if self.numrearlights == 0 then
			if self.movingDirection < 0 then
				self.worklightsActive = true;
			else
				self.worklightsActive = false;
			end;
		end;
	end;
	
	   if self.worklightsActive then
			for i=1, self.numWorklights do
				local worklight = self.worklights[i];
				setVisibility(worklight, self.worklightsActive);
			end;
		else
			for i=1, self.numWorklights do
				local worklight = self.worklights[i];
				setVisibility(worklight, self.worklightsActive, false);
			end;
		end;

	if self.numrearlights > 0 then
		if self.movingDirection < 0 then
			self.rearlightsActive = true;
--			self.worklightsActive = false;
		else
			self.rearlightsActive = false;
--			self.worklightsActive = true;
		end;
	
	   if self.rearlightsActive then
			for i=1, self.numrearlights do
				local rearlight = self.rearlights[i];
				setVisibility(rearlight, self.rearlightsActive);
			end;
		else
			for i=1, self.numrearlights do
				local rearlight = self.rearlights[i];
				setVisibility(rearlight, self.rearlightsActive, false);
			end;
		end;
	end;
end;

function TurnLightMotorized:draw()
end;

function TurnLightMotorized:delete()
end;